% Copyright (C) 2016  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% -------------------------------------
%
% [N, nSPperRegion, spOverlap] = resolveScheme(scheme)
% 
% Resolves a scheme-id: create parameter configuration from scheme-id
% e.g. [N, nSPperRegion, spOverlap] = resolveScheme(scheme) results in
%     nSPperRegion = 2
%     spOverlap = 1
%     N = [1,1;
%          2,2;
%          3,3;
%          4,4;
%          5,5];   
%
% This means there are 1+4+9+16+25=55 regions, each regions consists of
% 2x2 superpixels and neighbored regions overlap by 1 superpixel.
%
% scheme ... a string indicating the parameter set, e.g. '55_50'
% N ... n-by-2 matrix, each row: number of vertical and horizontal regions
% nSPperRegion ... number of horizontal and vertical superpixels of each
%                  region
% spOverlap ... number of overlaping superpixels
%
function [N, nSPperRegion, spOverlap] = resolveScheme(scheme)
  
  if strcmp(scheme, '55_50')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   

  elseif strcmp(scheme, '55_50_from4')
    nSPperRegion = 4;
    spOverlap = 2;
    
    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_50_from6')
    nSPperRegion = 6;
    spOverlap = 3;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_50_from8')
    nSPperRegion = 8;
    spOverlap = 4;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_75')
    nSPperRegion = 4;
    spOverlap = 3;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_25')
    nSPperRegion = 4;
    spOverlap = 1;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_10')
    nSPperRegion = 10;
    spOverlap = 1;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_0')
    nSPperRegion = 2;
    spOverlap = 0;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_90')
    nSPperRegion = 10;
    spOverlap = 9;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_80')
    nSPperRegion = 10;
    spOverlap = 8;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_70')
    nSPperRegion = 10;
    spOverlap = 7;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_60')
    nSPperRegion = 10;
    spOverlap = 6;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_50_from10')
    nSPperRegion = 10;
    spOverlap = 5;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_40')
    nSPperRegion = 10;
    spOverlap = 4;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_30')
    nSPperRegion = 10;
    spOverlap = 3;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_20')
    nSPperRegion = 10;
    spOverlap = 2;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_10_from10')
    nSPperRegion = 10;
    spOverlap = 1;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, '55_95')
    nSPperRegion = 20;
    spOverlap = 19;

    % y,x
    N = [1,1;
         2,2;
         3,3;
         4,4;
         5,5];   
  elseif strcmp(scheme, 'layer1')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [1,1];  
  elseif strcmp(scheme, 'layer2')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [2,2];  
  elseif strcmp(scheme, 'layer3')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [3,3];   
  elseif strcmp(scheme, 'layer4')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [4,4];   
  elseif strcmp(scheme, 'layer5')
    nSPperRegion = 2;
    spOverlap = 1;

    % y,x
    N = [5,5];   
  else
    error('Unknown scheme %s\n', scheme);  
  end
end