% Copyright (C) 2016  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% -------------------------------------
%
% L = segment_gridSeams(I, nx, ny, compactness, useInitialShift)
%
% Compute a compact segmentation with regular 4-nieghborhood.
% 
% nx, ny ... number of horizontal and vertical segments
% compactness .. compactness parameter (higher is more regular), in range [0,1], e.g. 0.75
% useInitialShift ... if 1, the highest probability of the seems is initialiy 
%   shifted to regions with high gradient (more stable in case of viewpoint changes)
% 
% L ... label image, all pixels of the segment share the same number in this image
%       The labels are in column-major order.
%
function L = segment_gridSeams(I, nx, ny, compactness, useInitialShift)
  
  if ~exist('useInitialShift', 'var')
    useInitialShift=1;
  end
  
  % prepare image
  G = rgb2gray(I);
  G = double(G)./255;

  % get vertical and horizontal seams 
  [BV, LV] = getVerticalSeams(G,nx-1, compactness, useInitialShift);
  [BH, LH] = getVerticalSeams(G',ny-1, compactness, useInitialShift);  
  
  % get combined label image
  L = (LV-1) * nx + (LH');
    
%   assert(nnz(L==0)==0);
%   assert(max(L(:))==(nx*ny));
%   assert(max(size(unique(L)))==(nx*ny));
  
end

% Split the image into nx vertical strips and find a single seam in each.
% The concatenated result will contain nx vertical seams
%
% G .. gray scale double image in range [0,1]
% nx ... the number of created vertical seams
function [B, L] = getVerticalSeams(G,nx, compactness, useInitialShift)

  if ~exist('useInitialShift', 'var')
    useInitialShift=0;
  end

  B = zeros(size(G));
  L = zeros(size(G));
  
  % split image
  [h, w, c] = size(G); 
  stepX = w/(nx+1);
  
  % get initial shift
  if useInitialShift
    sobelV = fspecial('sobel');
    sobelH = sobelV';
    W_gradient = abs(imfilter(G, sobelH, 'symmetric'));  
    initialShift = findBestHorizontalInitializationShift(W_gradient, nx);
  end
  
  % find each seam
  for ix = 1:nx
    
    % get image part
    startX = round( stepX/2 + (ix-1) * stepX + 1 );
    endX   = round( stepX/2 +    ix  * stepX     );
    startY = 1;
    endY = h;    
    
    if useInitialShift
      startX = max(1, startX+initialShift);       
      endX= min(endX+initialShift, w);
    end
    
    partI = G(startY:endY, startX:endX, :);
      
    % set up costs
    C = getCostMap(partI, compactness);

    % get path
    pix = getSeam(C);
   
    % plot boundary
    for i=1:numel(pix)
      B(startY+i-1, startX+pix(i)-1) = 1;
    end
  
    % update labels (left part is [ix], right part is [ix+1])
    for i=1:numel(pix)      
      L(startY+i-1, startX:startX+pix(i)-1) = ix;      
      L(startY+i-1, startX+pix(i):endX) = ix+1;
    end   
  
    % label first and last half-segments
    if ix==1
      for i=1:numel(pix)
        L(startY+i-1, 1:startX) = ix;
      end
    end    
    if ix==nx
      for i=1:numel(pix)
        L(startY+i-1, endX+1:end) = nx+1;
      end
    end
    
  end
  
end

% Compute costs for vertical split around the middle
% I ... image, double, [0,1]
function C = getCostMap(I, compactness)

  % Weight for gradient
  sobelV = fspecial('sobel');
  sobelH = sobelV';
  W_gradient = abs(imfilter(I, sobelH, 'symmetric'));  
  
  % A priori weight (higher probability in the middle)
  w = size(I,2);  
  sigma = (1-compactness)*w;
  gaus = fspecial('gaussian',[1 w], sigma);
  W_prior = repmat(gaus, size(I,1), 1);
  
  % Combine weights
  W = W_gradient .* W_prior;
  
  % Transform weights into costs  
  C = 1-W;
  
end

% Compute a vertical path with minimal cost
% 
% E.g., to plot the resulting path on color image I
%   for i=1:size(pix,1)
%     I(i,pix(i),:) = cat(3,255,0,0);
%   end
%
% C ... cost function
% pix ... list of pixels: pix(i) is the column of the i-th pixel on the
% path
%
function pix = getSeam(C)
  
  % factor on costs to penalize going diagonal instead of vertically
  % top-down (optional)
  diagonalPenalty = 1.000001;
  
  % This is a Dijkstra-like dynamic programming approach.
  % P will store the accumulated costs when going from top to this pixel. 
  % P(i,j) is the minimum cost of reaching P(i,j) either from the
  % left-above antecessor P(i-1,j-1), the direct above antecessor P(i-1,j)
  % or the right-above antecessor P(i-1,j+1).
  P = double(C);
  T = zeros(3, size(C,2));
  for i=2:size(C,1)
    T(1,:) = [inf P(i-1, 1:end-1)] .* diagonalPenalty; % left
    T(2,:) = [P(i-1, 2:end) inf] .* diagonalPenalty; % right
    T(3,:) = P(i-1, :); % above
    T_min = min(T);
    P(i,:) = P(i,:) + T_min;    
  end

  % Find the pixel with minium cost in the bottm row (this is the end point
  % of the minimum-cost path from top to bottom) and iteratively go upwards 
  % by following the possible antecessor (top-left, top, top-right pixel) 
  % with minimum cost.
  pix = zeros(size(C,1), 1);
  [~, pix(end)] = min(P(end, :));
  for i=size(C,1)-1:-1:1
    T_idx = [max(1, pix(i+1)-1) pix(i+1) min(size(P,2), pix(i+1)+1)];
    T_val = P(i, T_idx);
    [~,idx] = min(T_val);
            
    pix(i) = T_idx(idx);
  end

end