% Copyright (C) 2016  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% -------------------------------------
% 
% regions = getSPGridRegions(I, N, nSPperRegion, spOverlap, scale, spFct, spFctParams)
%
% Create Superpixel Grid regions by segmenting the image at different
% scales using a copmpact superpixel segmentation algorithm and
% subseqeuntly combining these superpixels into overlapping regions.
%
% N ... n-by-2 matrix, each row: number of vertical and horizontal regions 
% nSPperRegion ... number of horizontal and vertical superpixels of each
%                  region
% spOverlap ... number of overlaping superpixels
% scale ... factor for resizing the image
% spFct ... function handle of the compact superpixel segmentation
%           algorithm, the function is called this way:  
%           L = spFct(J,nSegX, nSegX, spFctParams);
% spFctParams ... parameters of the segmentation function
%
% SP-Grid parameters, e.g.:
%     nSPperRegion = 2;
%     spOverlap = 1;
%     N = [1,1;
%          2,2;
%          3,3;
%          4,4;
%          5,5];  
%
% This results in (1*1)+(2*2)+(3*3)+(4*4)+(5*5)=55 regions, each regions 
% consists of 2x2 superpixels and neighbored regions overlap by 1 
% superpixel.
%
% Example usage (exploiting the getSPGridRegionsScheme wrapper):
% params.compactness=0.75;
% regions = getSPGridRegionsScheme(I, '55_50', 0.5, @segment_gridSeamsWithParams, params);
%
function regions = getSPGridRegions(I, N, nSPperRegion, spOverlap, scale, spFct, spFctParams)

  if ~exist('scale', 'var')
   scale = 0.5;
  end
    
  % resize image  
  [h,w,c] = size(I);
  J = imresize(I,scale);
     
  % for each grid size, segment image and collect regions 
  idx=1;
  regions={};
  for i=1:size(N,1)
      
      % segment image
      nx = N(i,2);
      ny = N(i,1);
      
      nSegX = (nx*nSPperRegion) - (nx-1)*spOverlap;
      nSegY = (ny*nSPperRegion) - (ny-1)*spOverlap;
      
      % L has to be an ordered label image column major labels      
      L = spFct(J,nSegX, nSegX, spFctParams);      
            
      % collect bounding boxes for the labels
      n = max(L(:));
      clusters = zeros(6, n); % each col [x;y;minx;miny;maxx;maxy] 
      A = zeros(1,n); % area
      for j=1:n
          [y,x] = find(L==j);
          if ~isempty(x)
            xmin = min(x); 
            ymin = min(y); 
            xmax = max(x); 
            ymax = max(y);
            clusters(:,j) = [ mean(x); mean(y); xmin; ymin; xmax; ymax]; 
            A(j) = (xmax-xmin+1)*(ymax-ymin+1);
          else
            clusters(:,j) = nan(6,1); 
          end
      end
            
      % check clusters
      %clusters = checkClusters(clusters, nSegX, nSegX);
      
      
      % create regions by collecting (kxk) groups of segments
      for y=1:ny
        for x=1:nx

          % get sement labels
          % top left corner
          x1 = (x-1)*(nSPperRegion-spOverlap)+1;
          y1 = ((y-1)*(nSPperRegion-spOverlap))+1; 
          ll = [];
          for yy=y1:y1+nSPperRegion-1
            for xx=x1:x1+nSPperRegion-1
              label = (xx-1)*nSegY + yy;
              ll(end+1) = label;
            end
          end

          % get bounding box
          xmin = min( clusters(3, ll)  );
          xmax = max( clusters(5, ll)  );
          ymin = min( clusters(4, ll)  );
          ymax = max( clusters(6, ll)  );

          xmin = max(1, xmin*1/scale);
          ymin = max(1, ymin*1/scale);
          xmax = min(w, xmax*1/scale);
          ymax = min(h, ymax*1/scale);

          bb = [xmin ymin xmax ymax];

          M = zeros(size(L));
          for ii=1:numel(ll)
            M = M + (L==ll(ii));
          end
          M = imresize( M>0, 1/scale, 'nearest');

          % create region
          regions{idx}.ROI = bb;
          regions{idx}.patch = I(bb(2):bb(4), bb(1):bb(3), :);
          regions{idx}.mask = M(bb(2):bb(4), bb(1):bb(3), :);

          idx=idx+1;
        end
      end        
      
  end
end


% Check clustzer propertiesa and return a valid arangement if possitble
function clusters_valid = checkClusters(clusters, nx, ny)

  % check number of SP
  if size(clusters, 2)~=(nx*ny)
    warning('Not valid number of clusters: %d, nx=%d, ny=%d\n', size(clusters, 2), nx, ny);
  end
  
  meanx = clusters(1, :)';
  meany = clusters(2, :)';

  [~, meanx_sort_idx] = sort(meanx);
  
  clusters_valid = [];
  for x = 1:nx
    % take remaining ny left most    
    cur_idx = meanx_sort_idx(1:ny);
    meanx_sort_idx(1:ny) = [];
    
    % sort by y value
    cur_meany = meany(cur_idx);  % the y values of current clusters
    [~, cur_meany_sort_idx] = sort(cur_meany); % sort
    
    % store
    clusters_valid = [clusters_valid, clusters(:, cur_idx(cur_meany_sort_idx))];
    %cur_idx(cur_meany_sort_idx)
  end
  
  
end













